/*
 * Copyright (C) 1997-2012 Andre Noll <maan@systemlinux.org>
 *
 * Licensed under the GPL v2. For licencing details see COPYING.
 */

/** \file client.h Common client functions and exported symbols from client_common.c. */

/** The different states of a connection from the view of the client. */
enum {
	/** TCP connection is established. */
	CL_CONNECTED,
	/** Server sends the welcome message. */
	CL_RECEIVED_WELCOME,
	/** Client sends the authentification request. */
	CL_SENT_AUTH,
	/** Server sends encrypted challenge. */
	CL_RECEIVED_CHALLENGE,
	/** Client solves the challenge and sends the result. */
	CL_SENT_CH_RESPONSE,
	/** Server accepts this authentication. */
	CL_RECEIVED_PROCEED,
	/** Client sends the command. */
	CL_SENT_COMMAND,
	/** Server expects data. */
	CL_SENDING,
	/** Client expects data. */
	CL_RECEIVING,
};

/** Data specific to a client task. */
struct client_task {
	/** The state of the connection. */
	int status;
	/** The file descriptor and the session keys. */
	struct stream_cipher_context scc;
	/** True if this connections uses the sideband API. */
	bool use_sideband;
	/** The sideband context, ignored if \a use_sideband is false. */
	struct sb_context *sbc;
	/** The hash value of the decrypted challenge. */
	unsigned char *challenge_hash;
	/** The configuration (including the command). */
	struct client_args_info conf;
	/** The config file for client options. */
	char *config_file;
	/** The RSA private key. */
	char *key_file;
	/** Paraslash user name. */
	char *user;
	/** The client task structure. */
	struct task task;
	/** The buffer tree node of the client task. */
	struct btr_node *btrn;
	/** List of features supported by the server. */
	char **features;
};

void client_disconnect(struct client_task *ct);
void client_close(struct client_task *ct);
int client_parse_config(int argc, char *argv[], struct client_task **ct_ptr,
		int *loglevel);
int client_connect(struct client_task *ct, struct sched *s,
		struct btr_node *parent, struct btr_node *child);
int client_open(int argc, char *argv[], struct client_task **ct_ptr,
		int *loglevel, struct btr_node *parent, struct btr_node *child,
		struct sched *sched);
